import { __extends } from "tslib";
import { CronExpression, Expression, intervalExpr, rangeExpr, StringLiteral } from "../syntax/base";
/*eslint-disable @typescript-eslint/no-explicit-any */
var abstractVisitor = function (base) {
    var AbstractVisitor = /** @class */ (function (_super) {
        __extends(AbstractVisitor, _super);
        function AbstractVisitor() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var _this = _super.call(this, args) || this;
            _this.validateVisitor();
            return _this;
        }
        AbstractVisitor.prototype.cron = function (ctx) {
            return this.visit(ctx.cronExpression);
        };
        AbstractVisitor.prototype.cronExpression = function (ctx) {
            var visitedContext = new CronExpression();
            visitedContext.minute = this.visit(ctx.minutes);
            visitedContext.hour = this.visit(ctx.hours);
            visitedContext.dom = this.visit(ctx.dom);
            visitedContext.month = this.visit(ctx.month);
            visitedContext.dow = this.visit(ctx.dow);
            return visitedContext;
        };
        AbstractVisitor.prototype.expression = function (ctx) {
            var _this = this;
            var exprs = ctx.exprNotUnion.map(function (e) { return _this.visit(e); });
            return new Expression(exprs);
        };
        AbstractVisitor.prototype.exprNotUnion = function (ctx) {
            var lhs = new StringLiteral(ctx.lhs[0].image);
            return this.visit(ctx.atomicExpr, lhs);
        };
        AbstractVisitor.prototype.atomicExpr = function (ctx, lhs) {
            var expr = lhs;
            if (ctx.range) {
                var rhs = this.visit(ctx.range);
                var leftValue = Number(lhs.value());
                var rightValue = Number(rhs.value());
                if (leftValue !== NaN && rightValue !== NaN) {
                    if (leftValue > rightValue) {
                        throw new Error("Left-hand side range value must be smaller than right-hand side");
                    }
                }
                expr = rangeExpr(lhs, rhs);
            }
            if (ctx.interval) {
                expr = intervalExpr(expr, this.visit(ctx.interval));
            }
            return expr;
        };
        AbstractVisitor.prototype.interval = function (ctx) {
            return new StringLiteral(ctx.rhs[0].image);
        };
        AbstractVisitor.prototype.range = function (ctx) {
            return new StringLiteral(ctx.rhs[0].image);
        };
        return AbstractVisitor;
    }(base));
    return AbstractVisitor;
};
export default abstractVisitor;
